@@ -1,3 +1,4 @@
+import collections
 import importlib
 
 from click.testing import CliRunner
@@ -99,7 +100,7 @@ def test_grpc(  # type: ignore[misc]
     assert "import imports_service_pb2" not in service_module_types_text
     # this line should be in the code twice:
     # once for get_pb2, once for post_pb2
-    assert service_module_types_text.count("from . import requests") == 2
+    assert service_module_types_text.count("from . import requests") == 1
 
     # neither of the following two lines should be in the result
     assert "import requests.get_pb2" not in service_module_types_text
@@ -108,6 +109,38 @@ def test_grpc(  # type: ignore[misc]
     with grpc_imports.patched_syspath:
         importlib.import_module(f"{grpc_imports.package_name}.imports_service_pb2_grpc")
 
+    result = grpc_imports.generate(cli, args=["--in-place", "--create-package"])
+    assert result.exit_code == 0
+
+    assert all(
+        "__pycache__" not in p.parts for p in grpc_imports.package_dir.rglob("*.pyi")
+    )
+
+    assert all(
+        "__pycache__" not in line
+        for path in grpc_imports.package_dir.rglob("*.pyi")
+        for line in path.read_text().splitlines()
+    )
+
+    # the import only shows up in the grpc file because it's used in the
+    # service
+    counts = collections.Counter(
+        grpc_imports.package_dir.joinpath("imports_service_pb2_grpc.pyi")
+        .read_text()
+        .splitlines()
+    )
+
+    assert counts["from . import requests"] == 1
+
+    # notably it *does not* show up in the corresponding proto file unless the
+    # proto uses the message
+    counts = collections.Counter(
+        grpc_imports.package_dir.joinpath("imports_service_pb2.pyi")
+        .read_text()
+        .splitlines()
+    )
+    assert counts["from . import requests"] == 0
+
 
 def test_grpc_no_imports(  # type: ignore[misc]
     cli: CliRunner,
@@ -121,6 +154,11 @@ def test_grpc_no_imports(  # type: ignore[misc]
     result = no_imports_service.generate(cli, args=["--in-place", "--create-package"])
     assert result.exit_code == 0
 
+    init = no_imports_service.package_dir.joinpath("__init__.pyi")
+    init_lines = init.read_text().splitlines()
+    assert "from . import no_imports_service_pb2" in init_lines
+    assert "from . import no_imports_service_pb2_grpc" in init_lines
+
     service_module_types = no_imports_service.package_dir.joinpath(
         "no_imports_service_pb2_grpc.pyi"
     )
